/*
Copyright (C) 2012,2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.harchardware.comm;

import gnu.io.NoSuchPortException;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import org.harctoolbox.IrpMaster.IrpUtils;
import org.harctoolbox.harchardware.IStringCommand;

public final class LocalSerialPortBuffered extends LocalSerialPort implements IStringCommand {

    private BufferedReader bufferedInStream;

    public LocalSerialPortBuffered(String portName, int baud, int length, int stopBits, Parity parity, FlowControl flowControl, int timeout) throws NoSuchPortException, PortInUseException, UnsupportedCommOperationException, IOException {
        super(portName, baud, length, stopBits, parity, flowControl, timeout);
    }

    public LocalSerialPortBuffered(String portName, int baud) throws NoSuchPortException, PortInUseException, UnsupportedCommOperationException, IOException {
        this(portName, baud, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, Parity.NONE, FlowControl.NONE, 0);
    }

    public LocalSerialPortBuffered(String portName) throws NoSuchPortException, PortInUseException, UnsupportedCommOperationException, IOException {
        this(portName, 9600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, Parity.NONE, FlowControl.NONE, 0);
    }

    public LocalSerialPortBuffered(int portNumber) throws IOException, NoSuchPortException, PortInUseException, UnsupportedCommOperationException {
        this(getSerialPortName(portNumber), 9600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, Parity.NONE, FlowControl.NONE, 0);
    }

    @Override
    public void open() throws NoSuchPortException, PortInUseException, UnsupportedCommOperationException, IOException {
        super.open();
        bufferedInStream = new BufferedReader(new InputStreamReader(inStream, IrpUtils.dumbCharset));
    }

    @Override
    public void sendString(String cmd) throws IOException {
        sendBytes(cmd.getBytes(IrpUtils.dumbCharset));
    }

    //*@Override
    public void sendBytes(byte[] data) throws IOException {
        outStream.write(data);
    }

    public void sendBytes(byte[] data, int offset, int length) throws IOException {
        outStream.write(data, offset, length);
    }

    public void sendByte(byte b) throws IOException {
        outStream.write(b);
    }

    @Override
    public String readString() throws IOException {
        return bufferedInStream.readLine();
    }

    public static void main(String[] args) {
        ArrayList<String> names;
        try {
            names = getSerialPortNames(false);
            for (String name : names) {
                System.out.println(name);
            }

            LocalSerialPortBuffered port = null;

            port = new LocalSerialPortBuffered(defaultPort, 38400, 8, 1, Parity.EVEN, FlowControl.NONE, 10000);
            String cmd = "#QVR\r";
            port.sendString(cmd);
            System.out.println(port.readString());

            port.close();

        } catch (NoSuchPortException ex) {
            System.err.println(ex.getMessage());
        } catch (PortInUseException ex) {
            System.err.println(ex.getMessage());
        } catch (UnsupportedCommOperationException ex) {
            System.err.println(ex.getMessage());
        } catch (IOException ex) {
            System.err.println(ex.getMessage());
        }
    }
}
